;;; acronym.el --- Get DEFINITION of ACRONYM or store some.

;; Copyright (C) 2007  Andreas Röhler

;; Author: Andreas Röhler

;; Keywords: extensions

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
;; Boston, MA 02110-1301, USA.

;;; Commentary:

;; Idea came from Michael Olson's wtf-is.el,
;; acronym.el gladly uses its sorting algorithm.  

;; Acronym.el now has a facility to pick and store
;; acronyms together with their definition from your
;; browser: `acronym-add-current-kill-or-region'.

;; In order to store large amounts of acronyms,
;; collection of acronyms which came with "wtf-is.el"
;; now is stored in "acronym-alist-comp.el";
;; "~/acronym-alist-jur.el" and
;; "~/acronym-alist-lang.el" are delivered just as
;; examples.

;; Storing acronyms now is case-sensitive.  This was
;; important to grasp ingenious German legal terms as

;; "AbfStatErhTÄnd" => "Verordnung zur Änderung der
;; Erhebungstermine für die Abfallstatistiken".

;; For reading acronyms you may customize
;; `acronym-case-fold-search'.  Default is t.

;; Any comments welcome.

;;; Code:

(require 'string-strip)

;; provide compatibility with XEmacs
(defsubst region-exists-p ()
 "(not (null (mark)))"
 (not (null (mark))))

(defun region-active-p ()
  "and mark-active transient-mark-mode
 (not (eq (region-beginning) (region-end)"
  (if (featurep 'xemacs)
      'region-active-p
    (and mark-active transient-mark-mode
	 (not (eq (region-beginning) (region-end))))))

(defcustom  acronym-search-directoy t
 "Search directory given with `acronym-dir' for acronyms."

 :type 'boolean
 :group 'extensions)

(defcustom acronym-ask  t
 "Be asked before new acronym is stored.
Default is t"

:type 'boolean
:group 'extensions)

(defcustom acronym-search-all  t
 "If all known `acronym-files' shall be processed.
Default is t."

:type 'boolean
:group 'extensions)

(defcustom  acronym-case-fold-search t
 "If `acronym-is' should use `case-fold-search'.
Default is t"

 :type 'boolean
 :group 'extensions)

(defcustom acronym-files
  '("~/acronym-alist-jur.el" "~/acronym-alist-comp.el"  "~/acronym-alist-sprache.el")
  "*A list of files to read acronyms from."
  :type '(alist :value-type (repeat string))
  :group 'extensions)

(setq acronym-files (list "~/acronym-alist-jur.el" "~/acronym-alist-comp.el"  "~/acronym-alist-sprache.el"))
 
(defcustom acronym-default-file "~/acronym-alist-comp.el"
  "*File name of your personal arcron default dictionary, or nil.  Changing this file only takes affect after restart.  Call acronym- with universal argument to specify and store wanted source immediatly."
  :type '(file :value-type (repeat string))
  :group 'extensions)

(defcustom acronym-dir "~/"
  "*Directory to store your personal acronym dictionary, or nil."
  :type '(file :value-type (repeat string))
  :group 'extensions)

(defcustom acronym-prefix "acronym-alist-"
  "*First part of your personal acronym dictionary, or nil."
  :type '(file :value-type (repeat string))
  :group 'extensions)

(defvar acro-find-form '(buffer-substring-no-properties
				   (progn
				     (beginning-of-line)
				     (skip-chars-forward "^\"")
				     (point))
				   (progn
				     (end-of-line)
				     (skip-chars-backward "^\"")
				     (point))))

(defun acronym-find-intern (acronym ispec this-acronym-file no-space-insert mode-abbrevs-only &optional expand)
  "Acronym-is calls this function. "
  (let ((gefunden)
	(old-case-fold-search case-fold-search))
    (find-file this-acronym-file)
    (goto-char (point-min))
    (setq case-fold-search acronym-case-fold-search)
    (while (re-search-forward (concat "\"" "\\b" acronym "\\b" "\"") nil t 1)
      (setq gefunden
	    (concat gefunden
		    (eval acro-find-form) "\n")))
    (kill-buffer (current-buffer))
    (setq case-fold-search old-case-fold-search)
    gefunden))

(defun acronym-add2list (acronym-alist acro-new acro-definition acronym-var afile)
  (let ((acronym-alist (sort (cons (cons acro-new acro-definition)
				   acronym-alist)
			     #'(lambda (a b)
				 (string< (car a) (car b))))))
    acronym-alist))

(defun acronym-add (acronym definition &optional new-file)
  "Add ACRONYM and its DEFINITION to the list of associations.
Optional argument NEW-FILE specifies database."

  (interactive "sAcronym:\nsDefinition:\nP")
  (acronym-add-1 acronym definition new-file acronym-ask))

(defalias 'aac 'acronym-add-current-kill-or-region)

(defun acronym-add-current-kill-or-region (&optional new-file acronym-ask)
  "Add acronym from region if exists, otherwise take
current-kill. "
  (interactive "P\np")
  (let* (acro-def acro-new
		  (beg
		   (cond ((region-active-p)
			  (region-beginning))))
		  (end
		   (cond ((region-active-p)
			  (region-end))))
		  (alltext
		   (split-string (if (or (and beg end))
				     (string-strip (buffer-substring-no-properties beg end))
				   (substring-no-properties (current-kill 0))))))
    (dolist (ele alltext)
      (if (string-match "[\[(){}\]\\|[A-ZÖÜÄ][A-ZÖÜÄ]" ele)
	  (setq acro-new (string-strip ele "\\(\\)"  "\\(\\)"))
	(setq acro-def (concat acro-def ele " "))))
    (acronym-add-1 acro-new acro-def new-file acronym-ask)))

(defun acronym-add-from-region (beg end &optional new-file)
  "Add acronym, with NEW-FILE specify a file to store it.
Expects an active region with cursor over acronym."
  (interactive "r\nP")
  (save-excursion
    (let* ((acro-new (substring-no-properties (word-at-point)))
	   (acro-end (end-of-thing 'word))
	   (acro-beg (beginning-of-thing 'word))
	   (acro-first
	    (cond ((< acro-end end)
		   (buffer-substring-no-properties acro-end end))
		  (t (buffer-substring-no-properties beg (1- acro-beg)))))
	   (acro-def (string-strip acro-first " (" " )")))
      (acronym-add-1 acro-new acro-def new-file acronym-ask))))

(defun acronym-add-1 (acro-new acro-definition &optional new-file acronym-ask)
  (let*((afile
	 (cond ((or new-file acronym-ask)
		(completing-read "Store new acronym in file: " acronym-files nil nil (concat acronym-dir acronym-prefix)))
	       (t acronym-default-file)))
	(acronym-var (file-name-nondirectory (file-name-sans-extension afile)))
	(acronym-alist
	 (when (file-readable-p afile)
	   (load afile nil t)
	   (cdr (eval (intern-soft (file-name-sans-extension (file-name-nondirectory afile))))))))
    (setq acronym-default-file afile)
    (if (assoc acro-new acronym-alist)
	(when (y-or-n-p (concat (format "%s" (assoc acro-new acronym-alist)) " already in \"" acronym-var "\". Store anyway? "))
	  (acronym-write (acronym-add2list acronym-alist acro-new acro-definition acronym-var afile) afile acronym-var acro-definition))
      (acronym-write (acronym-add2list acronym-alist acro-new acro-definition acronym-var afile) afile acronym-var acro-definition)
      )))

(defun acronym-write (acronym-alist afile acronym-var acro-definition)
  "Writes the updated list of acronyms according to ACRONYM-ALIST."
  (set-buffer (get-buffer-create (file-name-nondirectory afile)))
  (switch-to-buffer (current-buffer))
  (erase-buffer)
  (insert (concat "(setq " acronym-var))
  (insert "\n(quote")
  (insert "\n(list ")
  (dolist (elem acronym-alist)
    (unless (bolp)
      (princ "\n" (current-buffer)))
    (prin1 elem (current-buffer))
    (indent-according-to-mode))
  (princ "\n)))" (current-buffer))
  (switch-to-buffer (current-buffer))
  (goto-char (point-min))
  (goto-char (point-max))
  (indent-region (point-min) (point-max))
  (write-file acronym-default-file)
  (search-backward acro-definition)
  (push-mark))

(defun acronym-at-point (&optional acronym-file ispec)
  "Provide the definition for ACRONYM at cursor position.
When called interactively, display the message \"ACRONYM is DEF\".
Otherwise, return DEF.
DEF refers to the definition associated with ACRONYM in `acronym-alist'.
With ACRONYM-FILE, specify a new acronym data source file"
  (interactive "P\np")
  (let* ((acr (word-at-point))
	 (acronym (or acr
		      (comment-string-strip (read-from-minibuffer "Acronym: ") t t))))
    (acronym-is acronym acronym-file ispec)))

(defun acronym-is (acronym &optional acronym-file ispec)
  "Provide the definition for ACRONYM.
When called interactively, display the message \"ACRONYM is DEF\".
Otherwise, return DEF.
DEF refers to the definition associated with ACRONYM in `acronym-alist'.
Optional argument ACRONYM-FILE selects the database."
  (interactive "sAcronym: \nP\np")
  (acronym-is-intern acronym acronym-file ispec))

(defun acronym-is-intern (acronym acronym-file ispec)
  (window-configuration-to-register ?s)
  (save-excursion
    (let ((acronym-current-file
	   (if acronym-file
	       (completing-read "acronym-file: "
				acronym-files nil t (concat acronym-dir acronym-prefix))
	     acronym-default-file))
	  found)
      (setq acronym-default-file acronym-current-file)
      (when (stringp acronym)
	(if (and (not acronym-file) acronym-search-all)
	    (dolist (this-a-file acronym-files)
	      (setq found
		    (concat found (acronym-find-intern acronym ispec this-a-file nil nil))))
	  (setq found (acronym-find-intern acronym t acronym-current-file nil nil))))
      (jump-to-register ?s)
      (if (string= "" found)
	  (error (concat "   No definition for \"" acronym "\" available!"))
	(when ispec (message "%s"  found)))
      found)))

(defun acronym-apropos (&optional ispec)
  "Search acronym-files for expression, which may be part of an
acronym or a definition. "
  (interactive "p")
  (let* ((old-case-fold-search case-fold-search)
	 (app (read-from-minibuffer "Acronym apropos: "))
	 (gefunden))
    (dolist (this-acronym-file acronym-files)
      (find-file this-acronym-file)
      (goto-char (point-min))
      (setq case-fold-search 1)
      (while (re-search-forward app nil t 1)
	(setq gefunden
	      (concat gefunden (list-atpt))))
      (kill-buffer (current-buffer))
      (setq case-fold-search old-case-fold-search)
      (when ispec (message "%s" gefunden))
      gefunden)))

(defconst acronym-version "1.0" "
Version number of this version of acronym.el.")

(defun acronym-version (&optional ispec) 
  "Print and/or return acronym-version"
  (interactive "p")
  (when ispec
    (message "%s" acronym-version))
  acronym-version)

(provide 'acronym)
;;; acronym.el ends here
